import 'dart:ui';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:sunny_platform_widgets/src/platform.dart';

import '../sunny_platform_widgets.dart';

const Color _kModalBarrierColor = CupertinoDynamicColor.withBrightness(
  color: Color(0x33000000),
  darkColor: Color(0x7A000000),
);

ShowPlatformDialog<T> defaultShowPlatformDialog<T>() => ({
      required BuildContext context,
      required WidgetBuilder builder,
      bool? barrierDismissible,
      RouteSettings? routeSettings,
      bool useRootNavigator = true,
      bool materialUseSafeArea = true,
      Color? materialBarrierColor = Colors.black54,
      String? barrierLabel,
      Constraints? constraints,
      dynamic extraOptions,
    }) {
      if (isMaterial(context)) {
        return showDialog<T>(
          context: context,
          builder: builder,
          barrierDismissible: barrierDismissible ?? true,
          routeSettings: routeSettings,
          useRootNavigator: useRootNavigator,
          useSafeArea: materialUseSafeArea,
          //child: , deprecated
          barrierColor: materialBarrierColor,
          barrierLabel: barrierLabel,
        );
      } else {
        return showCupertinoDialog<T>(
          context: context,
          builder: builder,
          routeSettings: routeSettings,
          useRootNavigator: useRootNavigator,
          barrierDismissible: barrierDismissible ?? false,
          barrierLabel: barrierLabel,
        );
      }
    };

/// Displays either the showModalBottomSheet for material
/// or showCupertinoModalPopup for cupertino
ShowPlatformModalSheet<T> defaultShowPlatformModalSheet<T>() => ({
      required BuildContext context,
      required WidgetBuilder builder,
      MaterialModalSheetData? material,
      CupertinoModalSheetData? cupertino,
      dynamic extraOptions,
    }) {
      if (isMaterial(context)) {
        return showModalBottomSheet<T>(
          context: context,
          builder: builder,
          backgroundColor: material?.backgroundColor,
          elevation: material?.elevation,
          shape: material?.shape,
          isScrollControlled: material?.isScrollControlled ?? false,
          useRootNavigator: material?.useRootNavigator ?? false,
          clipBehavior: material?.clipBehavior,
          barrierColor: material?.barrierColor,
          enableDrag: material?.enableDrag ?? true,
          isDismissible: material?.isDismissible ?? true,
          routeSettings: material?.routeSettings,
          transitionAnimationController:
              material?.transitionAnimationController,
        );
      } else {
        return showCupertinoModalPopup<T>(
          context: context,
          builder: builder,
          filter: cupertino?.imageFilter,
          semanticsDismissible: cupertino?.semanticsDismissible,
          useRootNavigator: cupertino?.useRootNavigator ?? true,
          barrierColor: cupertino?.barrierColor ?? _kModalBarrierColor,
          barrierDismissible: cupertino?.barrierDismissible ?? true,
          routeSettings: cupertino?.routeSettings,
        );
      }
    };

class MaterialModalSheetData {
  final Color? backgroundColor;
  final double? elevation;
  final ShapeBorder? shape;
  final bool isScrollControlled;
  final bool useRootNavigator;
  final Clip? clipBehavior;
  final Color? barrierColor;
  final bool enableDrag;
  final bool isDismissible;
  final RouteSettings? routeSettings;
  AnimationController? transitionAnimationController;

  MaterialModalSheetData({
    this.backgroundColor,
    this.elevation,
    this.shape,
    this.isScrollControlled = false,
    this.useRootNavigator = false,
    this.clipBehavior,
    this.barrierColor,
    this.enableDrag = false,
    this.isDismissible = false,
    this.routeSettings,
    this.transitionAnimationController,
  });
}

class CupertinoModalSheetData {
  final ImageFilter? imageFilter;
  final bool? semanticsDismissible;
  final bool useRootNavigator;
  final Color barrierColor;
  final bool barrierDismissible;
  final RouteSettings? routeSettings;

  CupertinoModalSheetData({
    this.imageFilter,
    this.semanticsDismissible,
    this.useRootNavigator = true,
    this.barrierColor = _kModalBarrierColor,
    this.barrierDismissible = true,
    this.routeSettings,
  });
}
